@@ -15,7 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  *
- * Authors: Carlos Garnacho Parro  <carlosg@gnome.org>
+ * Authors: Carlos Garnacho Parro  <carlosg@gnome.org>,
+ *          Milan Bouchet-Valat <nalimilan@club.fr>.
  */
 
 #include <dbus/dbus.h>
@@ -29,6 +30,7 @@
 #include "oobs-groupsconfig.h"
 #include "oobs-usersconfig.h"
 #include "oobs-group.h"
+#include "oobs-group-private.h"
 #include "oobs-defines.h"
 #include "utils.h"
 
@@ -39,7 +41,7 @@
  * @see_also: #OobsGroup
  **/
 
-#define GROUPS_CONFIG_REMOTE_OBJECT "GroupsConfig"
+#define GROUPS_CONFIG_REMOTE_OBJECT "GroupsConfig2"
 #define OOBS_GROUPS_CONFIG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OOBS_TYPE_GROUPS_CONFIG, OobsGroupsConfigPrivate))
 
 typedef struct _OobsGroupsConfigPrivate OobsGroupsConfigPrivate;
@@ -220,7 +222,7 @@ create_group_from_dbus_reply (OobsObject      *object,
 {
   OobsGroupsConfigPrivate *priv;
   DBusMessageIter iter;
-  int      gid;
+  guint32 gid;
   const gchar *groupname, *passwd;
   GList   *users;
   OobsGroup *group;
@@ -230,15 +232,15 @@ create_group_from_dbus_reply (OobsObject      *object,
 
   groupname = utils_get_string (&iter);
   passwd = utils_get_string (&iter);
-  gid = utils_get_int (&iter);
+  gid = utils_get_uint (&iter);
 
   users = utils_get_string_list_from_dbus_reply (reply, &iter);
 
-  group = g_object_new (OOBS_TYPE_GROUP,
-			"name", groupname,
-			"crypted-password", passwd,
-			"gid", gid,
-			NULL);
+  group = oobs_group_new (groupname);
+  g_object_set (G_OBJECT (group),
+                "password", passwd,
+                "gid", gid,
+                NULL);
 
   /* put the users list in the hashtable, will be used each time
    * the users config has changed, in order to get references to
@@ -251,96 +253,19 @@ create_group_from_dbus_reply (OobsObject      *object,
   return OOBS_GROUP (group);
 }
 
-static GList*
-get_users_list (OobsGroup *group)
-{
-  GList *users, *elem, *usernames = NULL;
-  OobsUser *user;
-
-  users = elem = oobs_group_get_users (group);
-
-  while (elem)
-    {
-      user = elem->data;
-      usernames = g_list_prepend (usernames, (gpointer) oobs_user_get_login_name (user));
-
-      elem = elem->next;
-    }
-
-  g_list_free (users);
-  return usernames;
-}
-
-static void
-create_dbus_struct_from_group (GObject         *group,
-			       DBusMessage     *message,
-			       DBusMessageIter *array_iter)
-{
-  DBusMessageIter struct_iter;
-  int    gid;
-  gchar *groupname, *passwd;
-  GList *users;
-
-  g_object_get (group,
-		"name", &groupname,
-		"crypted-password", &passwd,
-		"gid",  &gid,
-		NULL);
-
-  users = get_users_list (OOBS_GROUP (group));
-
-  dbus_message_iter_open_container (array_iter, DBUS_TYPE_STRUCT, NULL, &struct_iter);
-
-  utils_append_string (&struct_iter, groupname);
-  utils_append_string (&struct_iter, passwd);
-  utils_append_int (&struct_iter, gid);
-
-  utils_create_dbus_array_from_string_list (users, message, &struct_iter);
-
-  dbus_message_iter_close_container (array_iter, &struct_iter);
-
-  g_list_free (users);
-  g_free (groupname);
-  g_free (passwd);
-}
-
-static OobsUser*
-find_user (OobsList *users_list, gchar *username)
-{
-  OobsListIter iter;
-  OobsUser *user;
-  gboolean valid;
-
-  valid = oobs_list_get_iter_first (users_list, &iter);
-
-  while (valid)
-    {
-      user = OOBS_USER (oobs_list_get (users_list, &iter));
-
-      if (strcmp (username, oobs_user_get_login_name (user)) == 0)
-	return user;
-
-      valid = oobs_list_iter_next (users_list, &iter);
-      g_object_unref (user);
-    }
-
-  return NULL;
-}
-
 static void
 query_users_foreach (OobsGroup *group,
 		     GList     *users,
 		     gpointer   data)
 {
   OobsUsersConfig *users_config = OOBS_USERS_CONFIG (data);
-  OobsList *users_list = oobs_users_config_get_users (users_config);
   OobsUser *user;
 
   oobs_group_clear_users (group);
 
   while (users)
     {
-      user = find_user (users_list, users->data);
+      user = oobs_users_config_get_from_login (users_config, users->data);
 
       if (user)
 	{
@@ -395,8 +320,8 @@ oobs_groups_config_update (OobsObject *object)
 
   dbus_message_iter_next (&iter);
 
-  priv->minimum_gid = utils_get_int (&iter);
-  priv->maximum_gid = utils_get_int (&iter);
+  priv->minimum_gid = utils_get_uint (&iter);
+  priv->maximum_gid = utils_get_uint (&iter);
 
   users_config = oobs_users_config_get ();
 
@@ -423,22 +348,22 @@ oobs_groups_config_commit (OobsObject *object)
 
   dbus_message_iter_init_append (message, &iter);
   dbus_message_iter_open_container (&iter,
-				    DBUS_TYPE_ARRAY,
-				    DBUS_STRUCT_BEGIN_CHAR_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_TYPE_INT32_AS_STRING
-				    DBUS_TYPE_ARRAY_AS_STRING
-				    DBUS_TYPE_STRING_AS_STRING
-				    DBUS_STRUCT_END_CHAR_AS_STRING,
-				    &array_iter);
+                                    DBUS_TYPE_ARRAY,
+                                    DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+                                    DBUS_TYPE_STRING_AS_STRING
+                                    DBUS_TYPE_STRING_AS_STRING
+                                    DBUS_TYPE_UINT32_AS_STRING
+                                    DBUS_TYPE_ARRAY_AS_STRING
+                                    DBUS_TYPE_STRING_AS_STRING
+                                    DBUS_STRUCT_END_CHAR_AS_STRING,
+                                    &array_iter);
 
   valid  = oobs_list_get_iter_first (priv->groups_list, &list_iter);
 
   while (valid)
     {
       group = oobs_list_get (priv->groups_list, &list_iter);
-      create_dbus_struct_from_group (group, message, &array_iter);
+      _oobs_create_dbus_struct_from_group (OOBS_GROUP (group), message, &array_iter);
 
       g_object_unref (group);
       valid = oobs_list_iter_next (priv->groups_list, &list_iter);
@@ -483,3 +408,287 @@ oobs_groups_config_get_groups (OobsGroupsConfig *config)
 
   return priv->groups_list;
 }
+
+/**
+ * oobs_groups_config_add_group:
+ * @config: An #OobsGroupsConfig.
+ * @group: An #Oobsgroup.
+ *
+ * Add a group to the configuration, immediately committing changes to the system.
+ * On success, @group will be appended to the groups list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_groups_config_add_group (OobsGroupsConfig *config, OobsGroup *group)
+{
+  OobsGroupsConfigPrivate *priv;
+  OobsListIter list_iter;
+  OobsResult result;
+
+  g_return_val_if_fail (config != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (group != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_GROUPS_CONFIG (config), OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_GROUP (group), OOBS_RESULT_MALFORMED_DATA);
+
+  result = oobs_object_add (OOBS_OBJECT (group));
+
+  if (result != OOBS_RESULT_OK)
+    return result;
+
+  priv = config->_priv;
+
+  oobs_list_append (priv->groups_list, &list_iter);
+  oobs_list_set (priv->groups_list, &list_iter, G_OBJECT (group));
+
+  return OOBS_RESULT_OK;
+}
+
+/**
+ * oobs_groups_config_delete_group:
+ * @config: An #OobsGroupsConfig.
+ * @group: An #OobsGroup.
+ *
+ * Delete an group from the configuration, immediately committing changes to the system.
+ * On success, @group will be removed from the groups list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_groups_config_delete_group (OobsGroupsConfig *config, OobsGroup *group)
+{
+  OobsGroupsConfigPrivate *priv;
+  OobsGroup *list_group;
+  OobsListIter list_iter;
+  gboolean valid;
+  OobsResult result;
+
+  g_return_val_if_fail (config != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (group != NULL, OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_GROUPS_CONFIG (config), OOBS_RESULT_MALFORMED_DATA);
+  g_return_val_if_fail (OOBS_IS_GROUP (group), OOBS_RESULT_MALFORMED_DATA);
+
+  result = oobs_object_delete (OOBS_OBJECT (group));
+
+  if (result != OOBS_RESULT_OK)
+    return result;
+
+  priv = config->_priv;
+
+  valid = oobs_list_get_iter_first (priv->groups_list, &list_iter);
+
+  while (valid) {
+    list_group = OOBS_GROUP (oobs_list_get (priv->groups_list, &list_iter));
+
+    if (list_group == group)
+      break;
+
+    valid = oobs_list_iter_next (priv->groups_list, &list_iter);
+  }
+
+  oobs_list_remove (priv->groups_list, &list_iter);
+
+  return OOBS_RESULT_OK;
+}
+
+/**
+ * oobs_groups_config_get_from_name:
+ * @config: An #OobsGroupsConfig.
+ * @name: the name of the wanted group.
+ *
+ * Gets the (first) group called @name. This is a convenience function
+ * to avoid walking manually over the groups list.
+ *
+ * Return value: an #OobsGroup corresponding to the passed named,
+ * or %NULL if no such group exists. Don't forget to unref group when you're done.
+ **/
+OobsGroup *
+oobs_groups_config_get_from_name (OobsGroupsConfig *config, const gchar *name)
+{
+  OobsList *groups_list;
+  OobsListIter iter;
+  OobsGroup *group;
+  gboolean valid;
+  const gchar *group_name;
+
+  groups_list = oobs_groups_config_get_groups (config);
+  valid = oobs_list_get_iter_first (groups_list, &iter);
+
+  while (valid) {
+    group = OOBS_GROUP (oobs_list_get (groups_list, &iter));
+    group_name = oobs_group_get_name (group);
+
+    if (group_name && strcmp (name, group_name) == 0)
+      return group;
+
+    /* only the returned group is not unreferenced here */
+    g_object_unref (group);
+
+    valid = oobs_list_iter_next (groups_list, &iter);
+  }
+
+  return NULL;
+}
+
+/**
+ * oobs_groups_config_get_from_uid:
+ * @config: An #OobsGroupsConfig.
+ * @gid: the UID of the wanted group.
+ *
+ * Gets the (first) group whose GID is @gid. This is a convenience function
+ * to avoid walking manually over the groups list.
+ *
+ * Return value: an #OobsGroup corresponding to the passed GID,
+ * or %NULL if no such group exists. Don't forget to group user when you're done.
+ **/
+OobsGroup *
+oobs_groups_config_get_from_gid (OobsGroupsConfig *config, gid_t gid)
+{
+  OobsGroupsConfigPrivate *priv;
+  OobsGroup *group;
+  OobsListIter iter;
+  gboolean valid;
+  gid_t group_gid;
+
+  g_return_val_if_fail (config != NULL, NULL);
+  g_return_val_if_fail (OOBS_IS_GROUPS_CONFIG (config), NULL);
+
+  priv = config->_priv;
+
+  valid = oobs_list_get_iter_first (priv->groups_list, &iter);
+
+  while (valid) {
+    group = OOBS_GROUP (oobs_list_get (priv->groups_list, &iter));
+    group_gid = oobs_group_get_gid (group);
+
+    if (group_gid == gid)
+      return group;
+
+    /* only the returned group is not unreferenced here */
+    g_object_unref (group);
+
+    valid = oobs_list_iter_next (priv->groups_list, &iter);
+  }
+
+  return NULL;
+}
+
+/**
+ * oobs_groups_config_is_name_used:
+ * @config: An #OobsGroupsConfig.
+ * @name: the name to check.
+ *
+ * Check whether @name is already used by an existing group or not. This is
+ * a convenience function to avoid walking manually over the groups list.
+ *
+ * Return value: %TRUE if a group called @name already exists, %FALSE otherwise.
+ **/
+gboolean
+oobs_groups_config_is_name_used (OobsGroupsConfig *config, const gchar *name)
+{
+  OobsGroup *group;
+  gboolean name_used;
+
+  group = oobs_groups_config_get_from_name (config, name);
+  name_used = (group != NULL);
+
+  if (group)
+    g_object_unref (group);
+
+  return name_used;
+}
+
+/**
+ * oobs_groups_config_is_gid_used:
+ * @config: An #OobsGroupsConfig.
+ * @gid: the gid to check.
+ *
+ * Check whether @gid is already used by an existing group or not. This is
+ * a convenience function to avoid walking manually over the groups list.
+ *
+ * Return value: %TRUE if an group with such an gid already exists, %FALSE otherwise.
+ **/
+gboolean
+oobs_groups_config_is_gid_used (OobsGroupsConfig *config, gid_t gid)
+{
+  OobsGroup *group;
+  gboolean gid_used;
+
+  group = oobs_groups_config_get_from_gid (config, gid);
+  gid_used = (group != NULL);
+
+  if (group)
+    g_object_unref (group);
+
+  return gid_used;
+}
+
+
+/**
+ * oobs_groups_config_find_free_uid:
+ * @config: An #OobsGroupsConfig.
+ * @gid_min: the minimum wanted GID.
+ * @gid_max: the maximum wanted GID.
+ *
+ * Finds a GID that is not used by any user in the list. The returned GID is
+ * the highest used GID in the range plus one if @gid_max is not used.
+ * Else, the first free GID in the range is returned.
+ *
+ * If both @gid_min and @gid_max are equal to 0, the default range is used.
+ *
+ * Return value: a free GID in the requested range,
+ * or @gid_max to indicate wrong use or failure to find a free GID.
+ **/
+gid_t
+oobs_groups_config_find_free_gid (OobsGroupsConfig *config, gid_t gid_min, gid_t gid_max)
+{
+  OobsGroupsConfigPrivate *priv;
+  OobsList *list;
+  OobsListIter list_iter;
+  GObject *group;
+  gboolean valid;
+  gid_t new_gid, gid;
+
+  g_return_val_if_fail (config != NULL, gid_max);
+  g_return_val_if_fail (OOBS_IS_GROUPS_CONFIG (config), gid_max);
+  g_return_val_if_fail (gid_min <= gid_max, gid_max);
+
+  priv = config->_priv;
+
+  if (gid_min == 0 && gid_max == 0) {
+    gid_min = priv->minimum_gid;
+    gid_max = priv->maximum_gid;
+  }
+
+  new_gid = gid_min - 1;
+
+  list = oobs_groups_config_get_groups (config);
+  valid = oobs_list_get_iter_first (list, &list_iter);
+
+  /* Find the highest used GID in the range */
+  while (valid) {
+    group = oobs_list_get (list, &list_iter);
+    gid = oobs_group_get_gid (OOBS_GROUP (group));
+    g_object_unref (group);
+
+    if (gid < gid_max && gid >= gid_min && new_gid < gid)
+      new_gid = gid;
+
+    valid = oobs_list_iter_next (list, &list_iter);
+  }
+
+  new_gid++;
+
+  if (!oobs_groups_config_is_gid_used (config, new_gid))
+    return new_gid;
+
+
+  /* If the fast method failed, iterate over the whole range */
+  new_gid = gid_min;
+  while (oobs_groups_config_is_gid_used (config, new_gid) && new_gid < gid_max)
+    new_gid++;
+
+  /* In the extreme case where no GID is free in the range,
+   * we return the gid_max, which is the best we can do */
+  return new_gid;
+}
