@@ -61,6 +61,14 @@ struct _OobsObjectAsyncCallbackData
   gpointer data;
 };
 
+enum _OobsObjectCommitMethod
+{
+  METHOD_COMMIT,
+  METHOD_ADD,
+  METHOD_DELETE
+};
+typedef enum _OobsObjectCommitMethod _OobsObjectCommitMethod;
+
 static void oobs_object_class_init (OobsObjectClass *class);
 static void oobs_object_init       (OobsObject      *object);
 static void oobs_object_finalize   (GObject         *object);
@@ -114,6 +122,9 @@ oobs_object_class_init (OobsObjectClass *class)
 
   class->get_authentication_action = oobs_object_real_get_authentication_action;
 
+  /* some object types don't need to be singletons, they override this */
+  class->singleton = TRUE;
+
   dbus_connection_quark = g_quark_from_static_string ("oobs-dbus-connection");
 
   g_object_class_install_property (object_class,
@@ -156,7 +167,10 @@ oobs_object_init (OobsObject *object)
   OobsObjectPrivate *priv;
 
   priv = OOBS_OBJECT_GET_PRIVATE (object);
+
+  /* We need session to stay alive until object is destroyed */
   priv->session = oobs_session_get ();
+  g_object_ref (priv->session);
   priv->remote_object = NULL;
   dbus_error_init (&priv->dbus_error);
 
@@ -181,6 +195,9 @@ oobs_object_finalize (GObject *object)
 
   _oobs_session_unregister_object (priv->session, obj);
 
+  /* Only now, we don't care about session */
+  g_object_unref (priv->session);
+
   g_free (priv->remote_object);
   g_free (priv->path);
   g_free (priv->method);
@@ -194,10 +211,11 @@ oobs_object_constructor (GType                  type,
 			 guint                  n_construct_properties,
 			 GObjectConstructParam *construct_params)
 {
+  OobsObjectClass *class;
   static GHashTable *type_table;
   GObject *object;
 
-  /* objects that inherit from OobsObject have to be
+  /* some objects that inherit from OobsObject have to be
    * singletons, keep a hash table with references to them
    */
   if (!type_table)
@@ -205,15 +223,22 @@ oobs_object_constructor (GType                  type,
 
   object = g_hash_table_lookup (type_table, GINT_TO_POINTER (type));
 
-  if (!object)
+  if (object)
     {
-      object = (* G_OBJECT_CLASS (oobs_object_parent_class)->constructor) (type,
-									   n_construct_properties,
-									   construct_params);
-      g_hash_table_insert (type_table, GINT_TO_POINTER (type), object);
-      connect_object_to_session (OOBS_OBJECT (object));
+      class = OOBS_OBJECT_GET_CLASS (object);
+
+      /* if has to be singleton and exists, return it */
+      if (class->singleton)
+	return object;
     }
 
+  /* this class should not be limited to singletons, create (possibly new) object anyway */
+  object = (* G_OBJECT_CLASS (oobs_object_parent_class)->constructor) (type,
+                                                                       n_construct_properties,
+                                                                       construct_params);
+  g_hash_table_insert (type_table, GINT_TO_POINTER (type), object);
+  connect_object_to_session (OOBS_OBJECT (object));
+
   return object;
 }
 
@@ -488,11 +513,12 @@ run_message_async (OobsObject          *object,
 }
 
 static DBusMessage*
-get_commit_message (OobsObject *object)
+get_commit_message (_OobsObjectCommitMethod method, OobsObject *object)
 {
   OobsObjectClass   *class;
   OobsObjectPrivate *priv;
   DBusMessage       *message;
+  gchar *suffix;
 
   priv  = object->_priv;
   class = OOBS_OBJECT_GET_CLASS (object);
@@ -504,13 +530,30 @@ get_commit_message (OobsObject *object)
       return NULL;
     }
 
+
+  switch (method)
+    {
+      case METHOD_COMMIT:
+	suffix = "set";
+	break;
+      case METHOD_ADD:
+	suffix = "add";
+	break;
+      case METHOD_DELETE:
+	suffix = "del";
+	break;
+      default:
+	g_critical ("Unknown commit method");
+        return NULL;
+    }
+
   if (!class->commit)
     {
       g_critical ("There is no commit() implementation for this object");
       return NULL;
     }
 
-  message = dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, "set");
+  message = dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, suffix);
 
   /* Let the commit() implementation fill the message */
   _oobs_object_set_dbus_message (object, message);
@@ -544,57 +587,62 @@ get_update_message (OobsObject *object)
   return dbus_message_new_method_call (OOBS_DBUS_DESTINATION, priv->path, priv->method, "get");
 }
 
-/**
- * oobs_object_commit:
- * @object: an #OobsObject
- * 
- * Commits to the system all the changes done
- * to the configuration held by an #OobsObject.
- *
- * Return value: an #OobsResult enum with the error code.
- **/
-OobsResult
-oobs_object_commit (OobsObject *object)
+/*
+ * Do the real work for oobs_object_commit() oobs_object_add() and oobs_object_delete().
+ */
+static OobsResult
+do_commit (_OobsObjectCommitMethod method, OobsObject *object)
 {
+  OobsObjectPrivate *priv;
   DBusMessage *message;
+  DBusMessage *reply;
+  DBusMessageIter iter;
   OobsResult result;
 
   g_return_val_if_fail (OOBS_IS_OBJECT (object), OOBS_RESULT_MALFORMED_DATA);
 
-  message = get_commit_message (object);
+  message = get_commit_message (method, object);
 
   if (!message)
     return OOBS_RESULT_MALFORMED_DATA;
 
-  run_message (object, message, &result);
+  reply = run_message (object, message, &result);
   dbus_message_unref (message);
+
+  /* Objects can use this to update themselves when the backend provides it */
+  if (reply)
+    {
+      dbus_message_iter_init (reply, &iter);
+      if (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_STRUCT)
+	{
+	  priv = object->_priv;
+
+	  priv->update_requests++;
+	  result = update_object_from_message (object, reply);
+	  dbus_message_unref (reply);
+	}
+    }
+
   g_signal_emit (object, object_signals [COMMITTED], 0);
 
   return result;
 }
 
-/**
- * oobs_object_commit_async:
- * @object: An #OobsObject.
- * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
- * @data: Aditional data to pass to @func.
- * 
- * Commits to the system all the changes done to the configuration held by an #OobsObject.
- * This change will be asynchronous, being run the function @func when the change has been done.
- * 
- * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
- * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
- **/
-OobsResult
-oobs_object_commit_async (OobsObject          *object,
-			  OobsObjectAsyncFunc  func,
-			  gpointer             data)
+/*
+ * Do the real work for oobs_object_commit_async(),
+ * oobs_object_add_async() and oobs_object_delete_async().
+ */
+static OobsResult
+do_commit_async (_OobsObjectCommitMethod method,
+                 OobsObject             *object,
+                 OobsObjectAsyncFunc     func,
+                 gpointer                data)
 {
   DBusMessage *message;
 
   g_return_val_if_fail (OOBS_IS_OBJECT (object), OOBS_RESULT_MALFORMED_DATA);
 
-  message = get_commit_message (object);
+  message = get_commit_message (method, object);
 
   if (!message)
     return OOBS_RESULT_MALFORMED_DATA;
@@ -605,10 +653,117 @@ oobs_object_commit_async (OobsObject          *object,
   return OOBS_RESULT_OK;
 }
 
+/**
+ * oobs_object_commit:
+ * @object: an #OobsObject
+ *
+ * Commits to the system all the changes done
+ * to the configuration held by an #OobsObject.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_object_commit (OobsObject *object)
+{
+  return do_commit (METHOD_COMMIT, object);
+}
+
+/**
+ * oobs_object_commit_async:
+ * @object: An #OobsObject.
+ * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
+ * @data: Additional data to pass to @func.
+ *
+ * Commits to the system all the changes done to the configuration held by an #OobsObject.
+ * This change will be asynchronous, being run the function @func when the change has been done.
+ *
+ * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
+ * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
+ **/
+OobsResult
+oobs_object_commit_async (OobsObject          *object,
+			  OobsObjectAsyncFunc  func,
+			  gpointer             data)
+{
+  return do_commit_async (METHOD_COMMIT, object, func, data);
+}
+
+/**
+ * oobs_object_add:
+ * @object: an #OobsObject
+ *
+ * Add an #OobsObject to the system's configuration. This is only
+ * supported by certain individual objects, others are added by committing
+ * a modified objects list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_object_add (OobsObject *object)
+{
+  return do_commit (METHOD_ADD, object);
+}
+
+/**
+ * oobs_object_add_async:
+ * @object: An #OobsObject.
+ * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
+ * @data: Additional data to pass to @func.
+ *
+ * Add an #OobsObject to the system's configuration.
+ * This change will be asynchronous, being run the function @func when the change has been done.
+ *
+ * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
+ * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
+ **/
+OobsResult
+oobs_object_add_async (OobsObject          *object,
+                       OobsObjectAsyncFunc  func,
+                       gpointer             data)
+{
+  return do_commit_async (METHOD_ADD, object, func, data);
+}
+
+/**
+ * oobs_object_delete:
+ * @object: an #OobsObject
+ *
+ * Delete an #OobsObject from the system's configuration. This is only
+ * supported by certain individual objects, others are added by committing
+ * a modified objects list.
+ *
+ * Return value: an #OobsResult enum with the error code.
+ **/
+OobsResult
+oobs_object_delete (OobsObject *object)
+{
+  return do_commit (METHOD_DELETE, object);
+}
+
+/**
+ * oobs_object_delete_async:
+ * @object: An #OobsObject.
+ * @func: An #OobsObjectAsyncFunc that will be called when the asynchronous operation has ended.
+ * @data: Additional data to pass to @func.
+ *
+ * Delete an #OobsObject from the system's configuration.
+ * This change will be asynchronous, being run the function @func when the change has been done.
+ *
+ * Return value: an #OobsResult enum with the error code. Due to the asynchronous nature
+ * of the function, only OOBS_RESULT_MALFORMED and OOBS_RESULT_OK can be returned.
+ **/
+OobsResult
+oobs_object_delete_async (OobsObject          *object,
+                          OobsObjectAsyncFunc  func,
+                          gpointer             data)
+{
+  return do_commit_async (METHOD_DELETE, object, func, data);
+}
+
 /**
  * oobs_object_update:
  * @object: an #OobsObject
- * 
+ *
  * Synchronizes the configuration held by the #OobsObject
  * with the actual system configuration. All the changes done
  * to the configuration held by the #OobsObject will be forgotten.
